@@ -41,14 +41,15 @@ class UncertainValue:
             value [[val1, prob1], [val2, prob2], ...].
         n_samples (int): number of samples to be used for the calculation of
             the probability distribution. Default value is 100 000.
+        samples (list or numpy.ndarray): List/array with samples.
     """
 
     def __init__(self, nominal_value, lower_bound=None, upper_bound=None,
                  prob_distribution='uniform', distribution_param=None,
-                 n_samples=100000):
+                 n_samples=100000, samples=None):
         # # Assign values to self
         self.nominal_value = nominal_value
-        self.n_samples = n_samples
+        self.n_samples = int(n_samples)
         self.distribution_param = distribution_param
 
         # Bounds definition according to input
@@ -91,10 +92,19 @@ class UncertainValue:
             raise ValueError("Probability distribution type not recognized")
 
         # Samples
-        self.samples = self.__generate_samples(
-            self.lower_bound, self.upper_bound,
-            self.prob_distribution, distribution_param,
-            n_samples)
+        if samples is None:
+            self.samples = self.__generate_samples(
+                self.lower_bound, self.upper_bound,
+                self.prob_distribution, distribution_param,
+                self.n_samples)
+        elif samples is not None:
+            if isinstance(samples, (list, np.ndarray)):
+                self.prob_distribution = 'custom'
+                self.samples = samples
+                self.n_samples = len(samples)
+            else:
+                raise ValueError('Wrong format of samples. They should be '
+                                 + 'provided in a list or array.')
 
         # Compute statistical properties
         self.statistical_parameter()
@@ -234,7 +244,95 @@ class UncertainValue:
             other = UncertainValue(other)
         return other**self
 
-    # Redefine comparision
+    # Trigonometric functions
+    def sin(self):
+        """ Calculate the sine of the uncertain value.
+
+        Function is used as well when calling numpy.sin().
+
+        """
+        # Nominal value
+        nominal_value = np.sin(self.nominal_value)
+        # Bounds
+        bounds = [self.upper_bound, self.lower_bound]
+        delta = self.upper_bound - self.lower_bound
+        if 2 * np.pi <= delta:
+            lower_bound = -1.
+            upper_bound = 1.
+        elif (self.upper_bound - np.pi / 2) % (2 * np.pi) <= delta:
+            lower_bound = min(np.sin(bounds))
+            upper_bound = 1.
+        elif (self.upper_bound + np.pi / 2) % (2 * np.pi) <= delta:
+            lower_bound = -1.
+            upper_bound = max(np.sin(bounds))
+        else:
+            lower_bound = min(np.sin(bounds))
+            upper_bound = max(np.sin(bounds))
+        # Samples
+        samples = np.sin(self.samples)
+        # Results
+        result = UncertainValue(nominal_value, lower_bound, upper_bound,
+                                samples=samples)
+        return result
+
+    def cos(self):
+        """ Calculate the cosine of the uncertain value.
+
+        Function is used as well when calling numpy.cos().
+
+        """
+        # Nominal value
+        nominal_value = np.cos(self.nominal_value)
+        # Bounds
+        bounds = [self.upper_bound, self.lower_bound]
+        delta = self.upper_bound - self.lower_bound
+        if 2 * np.pi <= delta:
+            lower_bound = -1.
+            upper_bound = 1.
+        elif self.upper_bound % (2 * np.pi) <= delta:
+            lower_bound = min(np.cos(bounds))
+            upper_bound = 1.
+        elif (self.upper_bound + np.pi) % (2 * np.pi) <= delta:
+            lower_bound = -1.
+            upper_bound = max(np.cos(bounds))
+        else:
+            lower_bound = min(np.cos(bounds))
+            upper_bound = max(np.cos(bounds))
+        # Samples
+        samples = np.cos(self.samples)
+        # Results
+        result = UncertainValue(nominal_value, lower_bound, upper_bound,
+                                samples=samples)
+        return result
+
+    def tan(self):
+        """ Calculate the tangens of the uncertain value.
+
+        Function is used as well when calling numpy.tan().
+
+        """
+        # Nominal value
+        nominal_value = np.tan(self.nominal_value)
+        # Bounds
+        bounds = [self.upper_bound, self.lower_bound]
+        delta = self.upper_bound - self.lower_bound
+        if np.pi <= delta:
+            lower_bound = -np.inf
+            upper_bound = np.inf
+        elif (self.upper_bound - np.pi / 2) % np.pi <= delta:
+            lower_bound = -np.inf
+            upper_bound = np.inf
+        else:
+            lower_bound = min(np.tan(bounds))
+            upper_bound = max(np.tan(bounds))
+        # Samples
+        samples = np.tan(self.samples)
+        # Results
+        result = UncertainValue(nominal_value, lower_bound, upper_bound,
+                                samples=samples)
+        return result
+
+    # TODO: Redefine comparision ?
 
     # Other functions
     def describe(self):
